/*
 * Copyright 2010 Alibaba Group Holding Limited.
 * All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.alibaba.toolkit.util.configuration.digester;

import org.apache.commons.digester.Digester;
import org.apache.commons.digester.Rules;

/**
 * Ǵcommons-digesterչ<code>Digester</code>, һЩ, ԱʵصĹ.
 *
 * @version $Id: ContextSensitiveDigester.java,v 1.1 2003/07/03 07:26:16 baobao Exp $
 * @author Michael Zhou
 */
public class ContextSensitiveDigester extends Digester {
    /**
     * Ǹķ, <code>rules</code>װ<code>ContextSensitiveRules</code>.
     *
     * @param rules <code>Rules</code>
     */
    public void setRules(Rules rules) {
        if (!(rules instanceof ContextSensitiveRules)) {
            rules = new ContextSensitiveRules(rules);
        }

        this.rules = rules;
        this.rules.setDigester(this);
    }

    /**
     * Ǹ෽, ȷȡõĶΪ<code>ContextSensitiveRules</code>.
     *
     * @return <code>Rules</code>
     */
    public Rules getRules() {
        if (rules == null) {
            setRules(new ContextSensitiveRules());
        }

        return rules;
    }

    /**
     * <code>SetContextRule</code>, ʹָattributeֵΪǰcontextֵ.
     *
     * @param pattern       ǰƥ
     * @param attributeName XML
     */
    public void addSetContextRule(String pattern, String attributeName) {
        addRule(pattern, new SetContextRule(attributeName));
    }

    /**
     * <code>SetContextRule</code>, ʹָΪȡcontextĹ.
     *
     * @param pattern              ǰƥ
     * @param contextFactoryClass  
     */
    public void addSetContextRule(String pattern, Class contextFactoryClass) {
        addRule(pattern, new SetContextRule(contextFactoryClass));
    }

    /**
     * <code>SetContextRule</code>, ʹָcontextȡõǰcontextֵ.
     *
     * @param pattern        ǰƥ
     * @param contextFactory 
     */
    public void addSetContextRule(String pattern, ContextFactory contextFactory) {
        addRule(pattern, new SetContextRule(contextFactory));
    }

    /**
     * <code>SetRuleSetRule</code>, ʹָΪȡ<code>RuleSet</code>Ĺ.
     *
     * @param pattern              ǰƥ
     * @param ruleSetFactoryClass  
     */
    public void addSetRuleSetRule(String pattern, Class ruleSetFactoryClass) {
        addRule(pattern, new SetRuleSetRule(ruleSetFactoryClass));
    }

    /**
     * <code>SetRuleSetRule</code>, ʹָ<code>RuleSet</code>.
     *
     * @param pattern        ǰƥ
     * @param ruleSetFactory 
     */
    public void addSetRuleSetRule(String pattern, RuleSetFactory ruleSetFactory) {
        addRule(pattern, new SetRuleSetRule(ruleSetFactory));
    }
}
