/*
 * Copyright 2010 Alibaba Group Holding Limited.
 * All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.alibaba.toolkit.util.resourcebundle;

import java.io.BufferedInputStream;
import java.io.IOException;
import java.io.InputStream;

/**
 * <code>ResourceBundleFactory</code>, ʵͨõĴ<code>ResourceBundle</code>ʵķ.
 *
 * <p>
 * ʵͨ<code>ResourceBundleLoader</code>װbundleļ,
 * Ȼִ<code>parse</code>ȡ<code>ResourceBundle</code>ʵ.
 * </p>
 *
 * <p>
 * չͨṩʵ<code>ResourceBundleLoader</code>ıbundleװ뷽ʽ, ļϵͳݿװ.
 * ͨ<code>parse</code>, Ըıȡļĸʽ, <code>XMLResourceBundleFactory</code>XMLĸʽļ.
 * </p>
 *
 * @version $Id
 * @author Michael Zhou
 */
public abstract class AbstractResourceBundleFactory
        extends ResourceBundleFactory {
    private final ResourceBundleLoader loader;

    /**
     * factory, ʹõǰ̵߳context class loaderΪbundleװ.
     */
    public AbstractResourceBundleFactory() {
        this(new ClassLoaderResourceBundleLoader());
    }

    /**
     * factory, ʹָclass loaderΪbundleװ.
     *
     * @param classLoader װbundleclass loader
     */
    public AbstractResourceBundleFactory(ClassLoader classLoader) {
        this(new ClassLoaderResourceBundleLoader(classLoader));
    }

    /**
     * factory, ʹָloaderΪbundleװ
     *
     * @param loader bundleװ
     */
    public AbstractResourceBundleFactory(ResourceBundleLoader loader) {
        this.loader = loader;
    }

    /**
     * ȡ<code>ResourceBundleLoader</code>.
     *
     * @return loader
     */
    public ResourceBundleLoader getLoader() {
        return loader;
    }

    /**
     * <code>ResourceBundle</code>ʵ.
     *
     * @param bundleName  Ҫbundle
     *
     * @return ´<code>ResourceBundle</code>ʵ, ָbundle, 򷵻<code>null</code>
     *
     * @throws ResourceBundleCreateException ָbundleļ, bundleʵʧ, ļʽ
     */
    public ResourceBundle createBundle(String bundleName)
            throws ResourceBundleCreateException {
        InputStream stream   = null;
        String      filename = getFilename(bundleName);

        if (loader != null) {
            stream = loader.openStream(filename);
        }

        // ļ, 򷵻null.
        if (stream == null) {
            return null;
        }

        try {
            
            // @TODO: ˴URLΪsystem ID
            return parse(new BufferedInputStream(stream), filename);
        } finally {
            try {
                stream.close();
            } catch (IOException e) {
            }
        }
    }

    /**
     * bundleȡresourceļ.
     *
     * @param bundleName  bundle
     *
     * @return resource
     */
    protected String getFilename(String bundleName) {
        return bundleName.replace('.', '/');
    }

    /**
     * , д<code>ResourceBundle</code>.
     *
     * @param stream    
     * @param systemId  ׼ַ(һļ)
     *
     * @return resource bundle
     *
     * @throws ResourceBundleCreateException ʧ
     */
    protected abstract ResourceBundle parse(InputStream stream, String systemId)
            throws ResourceBundleCreateException;

    /**
     * ȽfactoryǷЧ.  ڵЧfactory, ͬbundle, <code>createBundle</code>,
     * ԵõЧbundleʵ.
     *
     * @param other ҪȽϵfactory
     *
     * @return Ч, 򷵻<code>true</code>
     */
    public boolean equals(Object other) {
        if (other == this) {
            return true;
        }

        try {
            AbstractResourceBundleFactory otherFactory = (AbstractResourceBundleFactory) other;

            return (loader == null)
                       ? (otherFactory.loader == null)
                       : loader.equals(otherFactory.loader);
        } catch (NullPointerException npe) {
            return false;
        } catch (ClassCastException cce) {
            return false;
        }
    }

    /**
     * ȡfactoryhashֵ, factoryЧ, ǵhashֵҲ.
     *
     * @return factoryhashֵ
     */
    public int hashCode() {
        return (loader == null) ? 0
                                : loader.hashCode();
    }
}
