/*
 * Copyright 2010 Alibaba Group Holding Limited.
 * All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.alibaba.toolkit.util.resourcebundle;

import com.alibaba.toolkit.util.collection.SoftHashMap;
import com.alibaba.toolkit.util.resourcebundle.xml.XMLResourceBundleFactory;

import java.lang.ref.SoftReference;

import java.text.MessageFormat;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Locale;
import java.util.Map;

/**
 * <code>ResourceBundle</code>ʵĹ.
 *
 * @version $Id: ResourceBundleFactory.java,v 1.1 2003/07/03 07:26:35 baobao Exp $
 * @author Michael Zhou
 */
public abstract class ResourceBundleFactory {
    /**
     * ʹָbundle, Ĭϵlocale, Ĭϵfactoryȡresource bundle. ĬϵfactoryǴ̵߳context class
     * loaderȡԴļ, XMLĸʽԴļ.
     *
     * @param baseName  bundleĻ
     *
     * @return resource bundle
     *
     * @throws MissingResourceException  ָbundleδҵ, 򴴽bundle
     */
    public static final ResourceBundle getBundle(String baseName) {
        return getBundle(baseName, null, (ResourceBundleFactory) null);
    }

    /**
     * ʹָbundle, ָlocale, Ĭϵfactoryȡresource bundle. ĬϵfactoryǴ̵߳context class
     * loaderȡԴļ, XMLĸʽԴļ.
     *
     * @param baseName  bundleĻ
     * @param locale    
     *
     * @return resource bundle
     *
     * @throws MissingResourceException  ָbundleδҵ, 򴴽bundle
     */
    public static final ResourceBundle getBundle(String baseName, Locale locale) {
        return getBundle(baseName, locale, (ResourceBundleFactory) null);
    }

    /**
     * ʹָbundle, ָlocale, Ĭϵfactoryȡresource bundle. ĬϵfactoryǴָclass loaderȡԴļ,
     * XMLĸʽԴļ.
     *
     * @param baseName    bundleĻ
     * @param locale      
     * @param classLoader class loader
     *
     * @return resource bundle
     *
     * @throws MissingResourceException  ָbundleδҵ, 򴴽bundle
     */
    public static final ResourceBundle getBundle(String baseName, Locale locale,
                                                 ClassLoader classLoader) {
        return getBundle(baseName, locale, new XMLResourceBundleFactory(classLoader));
    }

    /**
     * ʹָbundle, ָlocale, ָloader, Ĭϵfactoryȡresource bundle.
     * ĬϵfactoryǴָloaderȡԴļ, XMLĸʽԴļ.
     *
     * @param baseName  bundleĻ
     * @param locale    
     * @param loader    bundleװ
     *
     * @return resource bundle
     *
     * @throws MissingResourceException  ָbundleδҵ, 򴴽bundle
     */
    public static final ResourceBundle getBundle(String baseName, Locale locale,
                                                 ResourceBundleLoader loader) {
        return getBundle(baseName, locale, new XMLResourceBundleFactory(loader));
    }

    /**
     * ʹָbundle, ָlocale, ָfactoryȡresource bundle.
     *
     * @param baseName  bundleĻ
     * @param locale    
     * @param factory   bundle
     *
     * @return resource bundle
     *
     * @throws MissingResourceException  ָbundleδҵ, 򴴽bundle
     */
    public static final ResourceBundle getBundle(String baseName, Locale locale,
                                                 ResourceBundleFactory factory) {
        if (locale == null) {
            locale = Locale.getDefault();
        }

        if (factory == null) {
            factory = new XMLResourceBundleFactory();
        }

        return Helper.getBundleImpl(baseName, locale, factory);
    }

    /**
     * <code>ResourceBundle</code>ʵ.
     *
     * @param bundleName  Ҫbundle
     *
     * @return ´<code>ResourceBundle</code>ʵ, ָbundle, 򷵻<code>null</code>
     *
     * @throws ResourceBundleCreateException ָbundleļ, bundleʵʧ, ļʽ
     */
    public abstract ResourceBundle createBundle(String bundleName)
            throws ResourceBundleCreateException;

    /**
     * ж<code>ResourceBundleFactory</code>ǷЧ. ⽫Ϊ<code>ResourceBundle</code>cache.
     *
     * @param obj ҪȽϵһ
     *
     * @return Ч, 򷵻<code>true</code>
     */
    public abstract boolean equals(Object obj);

    /**
     * ȡhashֵ.  Ч<code>ResourceBundleFactory</code>Ӧþͬhashֵ.
     *
     * @return hashֵ
     */
    public abstract int hashCode();

    /**
     * Һʹbundle.
     */
    private static final class Helper {
        /** (factory, bundleName, defaultLocale)ӳ䵽bundlecache. ڴ治ʱ, cacheݻԶͷ. */
        private static final Cache cache = new Cache();

        /**
         * ʹָbundle, ָlocale, ָfactoryȡresource bundle.
         *
         * @param baseName  bundleĻ
         * @param locale    
         * @param factory   bundle
         *
         * @return resource bundle
         *
         * @throws MissingResourceException  ָbundleδҵ, 򴴽bundle
         */
        private static ResourceBundle getBundleImpl(String baseName, Locale locale,
                                                    ResourceBundleFactory factory) {
            if (baseName == null) {
                throw new NullPointerException(ResourceBundleConstant.RB_BASE_NAME_IS_NULL);
            }

            // ʹfactoryΪbundleδҵı, factoryGCյʱ, cacheӦҲԱ.
            final Object NOT_FOUND = factory;

            // cacheȡbundle.
            String bundleName   = baseName;
            String localeSuffix = locale.toString();

            if (localeSuffix.length() > 0) {
                bundleName += ("_" + localeSuffix);
            } else if (locale.getVariant().length() > 0) {
                // : new Locale("", "", "VARIANT").toString == ""
                bundleName += ("___" + locale.getVariant());
            }

            // ȡϵͳlocale, ע, ֵܱı, ÿִʱȡ.
            Locale defaultLocale = Locale.getDefault();

            Object lookup = cache.get(factory, bundleName, defaultLocale);

            if (NOT_FOUND.equals(lookup)) {
                throwResourceBundleException(true, baseName, locale, null);
            } else if (lookup != null) {
                return (ResourceBundle) lookup;
            }

            // ʼҲbundle.
            Object parent = NOT_FOUND;

            try {
                // base bundle.
                Object root = findBundle(factory, baseName, defaultLocale, baseName, null,
                                         NOT_FOUND);

                if (root == null) {
                    root = NOT_FOUND;
                    cache.put(factory, baseName, defaultLocale, root);
                }

                // Ҫ·, getBundle("baseName", new Locale("zh", "CN", "Variant")),
                // Ҫ·ΪbaseName_zh, baseName_zh_CN, baseName_zh_CN_Varient.
                final List names        = calculateBundleNames(baseName, locale);
                List       bundlesFound = new ArrayList(ResourceBundleConstant.MAX_BUNDLES_SEARCHED);

                // base bundleѾҵ, ·Ϊ.
                boolean foundInMainBranch = (!NOT_FOUND.equals(root) && names.size() == 0);

                if (!foundInMainBranch) {
                    parent = root;

                    for (int i = 0; i < names.size(); i++) {
                        bundleName = (String) names.get(i);
                        lookup     = findBundle(factory, bundleName, defaultLocale, baseName,
                                                parent, NOT_FOUND);
                        bundlesFound.add(lookup);

                        if (lookup != null) {
                            parent            = lookup;
                            foundInMainBranch = true;
                        }
                    }
                }


                // ·δҵbundle, ϵͳĬ·, 統ǰϵͳĬlocaleΪen_US,
                // ·Ϊ: baseName_en, baseName_US.
                parent = root;

                if (!foundInMainBranch) {
                    final List fallbackNames = calculateBundleNames(baseName, defaultLocale);

                    for (int i = 0; i < fallbackNames.size(); i++) {
                        bundleName = (String) fallbackNames.get(i);

                        // ϵͳĬ··һ, Ҫȥ
                        if (names.contains(bundleName)) {
                            break;
                        }

                        lookup = findBundle(factory, bundleName, defaultLocale, baseName, parent,
                                            NOT_FOUND);

                        if (lookup != null) {
                            parent = lookup;
                        } else {
                            // bundleݸbundle, :
                            // bundle: baseName_en.xmlѾҵ, bundle: baseName_en_USδҵ,
                            // cache:
                            // baseName       => bundle: baseName.xml
                            // baseName_en    => bundle: baseName_en.xml
                            // baseName_en_US => bundle: baseName_en.xml
                            cache.put(factory, bundleName, defaultLocale, parent);
                        }
                    }
                }

                // ·, bundleݸbundle, :
                // 1. bundle·, getBundle("baseName", new Locale("zh", "CN")),
                //    baseName_zhҵ, cache:
                //    baseName       => bundle: baseName.xml
                //    baseName_zh    => bundle: baseName_zh.xml
                //    baseName_zh_CN => bundle: baseName_zh.xml
                //
                // 2. bundleϵͳ·, ·δҵ, getBundle("baseName", new Locale("zh", "CN")),
                //    baseName_zhbaseName_zh_CNδҵ, ϵͳ·baseName_enҵ, cache:
                //    baseName       => bundle: baseName.xml
                //    baseName_zh    => bundle: baseName_en.xml
                //    baseName_zh_CN => bundle: baseName_en.xml
                //    baseName_en    => bundle: baseName_en.xml
                //    baseName_en_US => bundle: baseName_en.xml
                //
                // 3. bundleĻδҵ:
                //    baseName       => NOT_FOUND
                //    baseName_zh    => NOT_FOUND
                //    baseName_zh_CN => NOT_FOUND
                //    baseName_en    => NOT_FOUND
                //    baseName_en_US => NOT_FOUND
                for (int i = 0; i < names.size(); i++) {
                    final String name        = (String) names.get(i);
                    final Object bundleFound = bundlesFound.get(i);

                    if (bundleFound == null) {
                        cache.put(factory, name, defaultLocale, parent);
                    } else {
                        parent = bundleFound;
                    }
                }
            } catch (Exception e) {
                // ResourceBundleCreateExceptionRuntimeException.
                cache.cleanUpConstructionList();
                throwResourceBundleException(false, baseName, locale, e);
            } catch (Error e) {
                cache.cleanUpConstructionList();
                throw e;
            }

            if (NOT_FOUND.equals(parent)) {
                throwResourceBundleException(true, baseName, locale, null);
            }

            return (ResourceBundle) parent;
        }

        /**
         * cacheвbundle, factoryװbundle.  ˷<code>null</code>, ߱Լbundle,
         * <code>cache.put</code>.
         *
         * @param factory       bundle
         * @param bundleName    bundle
         * @param defaultLocale ϵͳĬϵlocale
         * @param baseName      bundle
         * @param parent        bundle, ڸbundle,  Ϊ<code>null</code>
         * @param NOT_FOUND     "δҵ"״̬Ķ
         *
         * @return resource bundle, <code>null</code>ʾbundleδҵ
         *
         * @throws ResourceBundleCreateException bundleҵ, 첻ɹ
         */
        private static Object findBundle(ResourceBundleFactory factory, String bundleName,
                                         Locale defaultLocale, String baseName, Object parent,
                                         final Object NOT_FOUND)
                throws ResourceBundleCreateException {
            Object result = cache.getWait(factory, bundleName, defaultLocale);

            if (result != null) {
                return result;
            }


            // Դfactoryװbundle.
            result = factory.createBundle(bundleName);

            if (result != null) {
                // ڵfactoryʱ, пܵݹصgetBundle, bundleѾ.
                // , bundleһcache.  Ϊһ, Ӧcacheеbundle.
                Object otherBundle = cache.get(factory, bundleName, defaultLocale);

                if (otherBundle != null) {
                    result = otherBundle;
                } else {
                    // bundleĸbundle, ŵcache.
                    final ResourceBundle bundle = (ResourceBundle) result;

                    if ((!NOT_FOUND.equals(parent)) && (bundle.getParent() == null)) {
                        bundle.setParent((ResourceBundle) parent);
                    }

                    bundle.setBaseName(baseName);
                    bundle.setLocale(baseName, bundleName);
                    cache.put(factory, bundleName, defaultLocale, result);
                }
            }

            return result;
        }

        /**
         * ȡñѡbundle.
         *
         * @param baseName bundleĻ
         * @param locale   
         *
         * @return бѡbundle
         */
        private static List calculateBundleNames(String baseName, Locale locale) {
            final List   result         = new ArrayList(ResourceBundleConstant.MAX_BUNDLES_SEARCHED);
            final String language       = locale.getLanguage();
            final int    languageLength = language.length();
            final String country        = locale.getCountry();
            final int    countryLength  = country.length();
            final String variant        = locale.getVariant();
            final int    variantLength  = variant.length();

            // locale("", "", "").
            if ((languageLength + countryLength + variantLength) == 0) {
                return result;
            }

            final StringBuffer buffer = new StringBuffer(baseName);


            // baseName_language
            buffer.append('_');
            buffer.append(language);

            if (languageLength > 0) {
                result.add(buffer.toString());
            }

            if ((countryLength + variantLength) == 0) {
                return result;
            }


            // baseName_language_country
            buffer.append('_');
            buffer.append(country);

            if (countryLength > 0) {
                result.add(buffer.toString());
            }

            if (variantLength == 0) {
                return result;
            }


            // baseName_language_country_variant
            buffer.append('_');
            buffer.append(variant);
            result.add(buffer.toString());

            return result;
        }

        /**
         * "resource bundleδҵ"쳣.
         *
         * @param missing  ָbundleδҵ, Ǵbundle
         * @param baseName δҵbundle
         * @param locale   δҵbundle
         * @param cause    쳣
         */
        private static void throwResourceBundleException(boolean missing, String baseName,
                                                         Locale locale, Throwable cause) {
            String bundleName = baseName + "_" + locale;

            if (missing) {
                throw new ResourceBundleException(ResourceBundleConstant.RB_MISSING_RESOURCE_BUNDLE,
                                                  new Object[] {
                    baseName,
                    locale
                }, cause, bundleName, "");
            } else {
                throw new ResourceBundleException(ResourceBundleConstant.RB_FAILED_LOADING_RESOURCE_BUNDLE,
                                                  new Object[] {
                    baseName,
                    locale
                }, cause, bundleName, "");
            }
        }
    }

    /**
     * (factory, bundleName, defaultLocale)ӳ䵽bundlecache.
     */
    private static final class Cache extends SoftHashMap {
        /** ̬key, cacheвbundle. ʹþ̬ԼGCĸ. ʹcacheKeycacheͬ. */
        private static final CacheKey cacheKey = new CacheKey();

        /**
         * hashͬ߳, Աͬʱװͬһbundle. hashcacheKeythreadӳ. ʹôhashcacheͬ.
         */
        private final Map underConstruction = new HashMap(
                                                      ResourceBundleConstant.MAX_BUNDLES_SEARCHED,
                                                      ResourceBundleConstant.CACHE_LOAD_FACTOR);

        /**
         * һcache.
         */
        public Cache() {
            super(ResourceBundleConstant.INITIAL_CACHE_SIZE,
                  ResourceBundleConstant.CACHE_LOAD_FACTOR);
        }

        /**
         * cacheвbundle.
         *
         * @param factory       bundle
         * @param bundleName    bundle
         * @param defaultLocale ϵͳlocale
         *
         * @return cachebundle. δҵ, 򷵻<code>null</code>
         */
        public synchronized Object get(ResourceBundleFactory factory, String bundleName,
                                       Locale defaultLocale) {
            cacheKey.set(factory, bundleName, defaultLocale);

            Object result = get(cacheKey);

            cacheKey.clear();
            return result;
        }

        /**
         * cacheвbundle, bundle, һ߳ڹbundle, ȴ֮. ˷<code>null</code>,
         * ߱븺<code>put</code><code>cleanUpConstructionList</code>, ߳̿ܵȴ, .
         *
         * @param factory       bundle
         * @param bundleName    bundle
         * @param defaultLocale ϵͳlocale
         *
         * @return cachebundle. δҵ, 򷵻<code>null</code>
         */
        public synchronized Object getWait(ResourceBundleFactory factory, String bundleName,
                                           Locale defaultLocale) {
            Object result;


            // ȲcacheǷѾbundle, , ֱӷ.
            cacheKey.set(factory, bundleName, defaultLocale);
            result = get(cacheKey);

            if (result != null) {
                cacheKey.clear();
                return result;
            }

            // ǲѾһthreadڴbundle.
            // ע, пܵݹgetBundle, , factoryеgetBundle.
            // , beingBuilt == false
            Thread  builder    = (Thread) underConstruction.get(cacheKey);
            boolean beingBuilt = (builder != null && builder != Thread.currentThread());

            // Ѿһthreadڴbundle.
            if (beingBuilt) {
                while (beingBuilt) {
                    cacheKey.clear();

                    try {
                        // ȴ, ̴ֱ߳.
                        wait();
                    } catch (InterruptedException e) {
                    }

                    cacheKey.set(factory, bundleName, defaultLocale);
                    beingBuilt = underConstruction.containsKey(cacheKey);
                }


                // һ̰߳bundle, ֱӷؼ
                result = get(cacheKey);

                if (result != null) {
                    cacheKey.clear();
                    return result;
                }
            }


            // bundlecache, ׼bundle.
            // ߱putcleanUpConstructionList, 򽫻.
            underConstruction.put(cacheKey.clone(), Thread.currentThread());

            cacheKey.clear();

            return null;
        }

        /**
         * bundlecache, еȴ߳.
         *
         * @param factory       bundle
         * @param bundleName    bundle
         * @param defaultLocale ϵͳlocale
         * @param bundle        cachebundle
         */
        public synchronized void put(ResourceBundleFactory factory, String bundleName,
                                     Locale defaultLocale, Object bundle) {
            cacheKey.set(factory, bundleName, defaultLocale);

            put(cacheKey.clone(), bundle);

            underConstruction.remove(cacheKey);

            cacheKey.clear();


            // ߳
            notifyAll();
        }

        /**
         * "ڹbundle"̱߳ǰ߳. װbundleʧ, Ҫô˷.
         */
        public synchronized void cleanUpConstructionList() {
            final Collection entries    = underConstruction.values();
            final Thread     thisThread = Thread.currentThread();

            while (entries.remove(thisThread)) {
            }


            // ߳
            notifyAll();
        }
    }

    /**
     * bundleӦcache key, bundle, bundle, ϵͳlocaleֶ.
     */
    private static final class CacheKey implements Cloneable {
        private SoftReference factoryRef;
        private String        bundleName;
        private Locale        defaultLocale;
        private int           hashCode;

        /**
         * cache key.
         *
         * @param factory       bundle
         * @param bundleName    bundle
         * @param defaultLocale ϵͳlocale
         */
        public void set(ResourceBundleFactory factory, String bundleName, Locale defaultLocale) {
            this.bundleName    = bundleName;
            this.hashCode      = bundleName.hashCode();
            this.defaultLocale = defaultLocale;

            if (defaultLocale != null) {
                hashCode ^= defaultLocale.hashCode();
            }

            if (factory == null) {
                this.factoryRef = null;
            } else {
                factoryRef = new SoftReference(factory);
                hashCode ^= factory.hashCode();
            }
        }

        /**
         * cache key.
         */
        public void clear() {
            set(null, "", null);
        }

        /**
         * keyǷƥ.
         *
         * @param other  һcache key
         *
         * @return ƥ, 򷵻<code>true</code>
         */
        public boolean equals(Object other) {
            if (this == other) {
                return true;
            }

            try {
                final CacheKey otherKey = (CacheKey) other;

                // hashֵͬ, 
                if (hashCode != otherKey.hashCode) {
                    return false;
                }

                // bundleǷͬ?
                if (!eq(bundleName, otherKey.bundleName)) {
                    return false;
                }

                // localeǷͬ
                if (!eq(defaultLocale, otherKey.defaultLocale)) {
                    return false;
                }

                // factoryǷͬ?
                if (factoryRef == null) {
                    return otherKey.factoryRef == null;
                } else {
                    return (otherKey.factoryRef != null)
                           && eq(factoryRef.get(), otherKey.factoryRef.get());
                }
            } catch (NullPointerException e) {
                return false;
            } catch (ClassCastException e) {
                return false;
            }
        }

        /**
         * ȽǷ.
         *
         * @param o1 1
         * @param o2 2
         *
         * @return , 򷵻<code>true</code>
         */
        private boolean eq(Object o1, Object o2) {
            return (o1 == null) ? (o2 == null)
                                : o1.equals(o2);
        }

        /**
         * ȡhashֵ, Ч, hashֵҲ.
         *
         * @return hashֵ
         */
        public int hashCode() {
            return hashCode;
        }

        /**
         * ƶ.
         *
         * @return cache keyĸ
         */
        public Object clone() {
            try {
                return super.clone();
            } catch (CloneNotSupportedException e) {
                throw new InternalError(MessageFormat.format(
                                                ResourceBundleConstant.RB_CLONE_NOT_SUPPORTED,
                                                new Object[] {
                    CacheKey.class.getName()
                }));
            }
        }

        /**
         * ȡֵַʾ.
         *
         * @return ַʾ
         */
        public String toString() {
            return new StringBuffer("CacheKey[factory=").append((factoryRef == null)
                                                                    ? "null"
                                                                    : factoryRef.get())
                                                        .append(", bundleName=").append(bundleName)
                                                        .append(", defaultLocale=")
                                                        .append(defaultLocale).append("]")
                                                        .toString();
        }
    }
}
